JasonWoof Got questions, comments, patches, etc.? Contact Jason Woofenden
Jukka also belongs to Copyright holders after all he has contributed and done for...
[dwm.git] / layout.c
index 5a32fd3..7b9b3ef 100644 (file)
--- a/layout.c
+++ b/layout.c
@@ -1,5 +1,8 @@
 /* © 2006-2007 Anselm R. Garbe <garbeam at gmail dot com>
  * © 2006-2007 Sander van Dijk <a dot h dot vandijk at gmail dot com>
+ * © 2006-2007 Jukka Salmi <jukka at salmi dot ch>
+ * © 2007 Premysl Hruby <dfenze at gmail dot com>
+ * © 2007 Szabolcs Nagy <nszabolcs at gmail dot com>
  * See LICENSE file for license details. */
 #include "dwm.h"
 #include <stdlib.h>
@@ -14,6 +17,22 @@ static unsigned int masterw = MASTERWIDTH;
 static unsigned int nmaster = NMASTER;
 
 static void
+ban(Client *c) {
+       if (c->isbanned)
+               return;
+       XMoveWindow(dpy, c->win, c->x + 2 * sw, c->y);
+       c->isbanned = True;
+}
+
+static void
+unban(Client *c) {
+       if (!c->isbanned)
+               return;
+       XMoveWindow(dpy, c->win, c->x, c->y);
+       c->isbanned = False;
+}
+
+static void
 tile(void) {
        unsigned int i, n, nx, ny, nw, nh, mw, mh, tw, th;
        Client *c;
@@ -28,9 +47,7 @@ tile(void) {
 
        for(i = 0, c = clients; c; c = c->next)
                if(isvisible(c)) {
-                       if(c->isbanned)
-                               XMoveWindow(dpy, c->win, c->x, c->y);
-                       c->isbanned = False;
+                       unban(c);
                        if(c->isfloating)
                                continue;
                        c->ismax = False;
@@ -38,30 +55,31 @@ tile(void) {
                        ny = way;
                        if(i < nmaster) {
                                ny += i * mh;
-                               nw = mw - 2 * BORDERPX;
-                               nh = mh - 2 * BORDERPX;
+                               nw = mw - 2 * c->border;
+                               nh = mh;
+                               if(i + 1 == (n < nmaster ? n : nmaster)) /* remainder */
+                                       nh = wah - mh * i;
+                               nh -= 2 * c->border;
                        }
                        else {  /* tile window */
                                nx += mw;
-                               nw = tw - 2 * BORDERPX;
-                               if(th > 2 * BORDERPX) {
+                               nw = tw - 2 * c->border;
+                               if(th > 2 * c->border) {
                                        ny += (i - nmaster) * th;
-                                       nh = th - 2 * BORDERPX;
+                                       nh = th;
+                                       if(i + 1 == n) /* remainder */
+                                               nh = wah - th * (i - nmaster);
+                                       nh -= 2 * c->border;
                                }
-                               else /* fallback if th <= 2 * BORDERPX */
-                                       nh = wah - 2 * BORDERPX;
+                               else /* fallback if th <= 2 * c->border */
+                                       nh = wah - 2 * c->border;
                        }
                        resize(c, nx, ny, nw, nh, False);
                        i++;
                }
-               else {
-                       c->isbanned = True;
-                       XMoveWindow(dpy, c->win, c->x + 2 * sw, c->y);
-               }
-       if(!sel || !isvisible(sel)) {
-               for(c = stack; c && !isvisible(c); c = c->snext);
-               focus(c);
-       }
+               else
+                       ban(c);
+       focus(NULL);
        restack();
 }
 
@@ -73,22 +91,16 @@ void
 floating(void) {
        Client *c;
 
-       for(c = clients; c; c = c->next) {
+       for(c = clients; c; c = c->next)
                if(isvisible(c)) {
                        if(c->isbanned)
                                XMoveWindow(dpy, c->win, c->x, c->y);
                        c->isbanned = False;
                        resize(c, c->x, c->y, c->w, c->h, True);
                }
-               else {
-                       c->isbanned = True;
-                       XMoveWindow(dpy, c->win, c->x + 2 * sw, c->y);
-               }
-       }
-       if(!sel || !isvisible(sel)) {
-               for(c = stack; c && !isvisible(c); c = c->snext);
-               focus(c);
-       }
+               else
+                       ban(c);
+       focus(NULL);
        restack();
 }
 
@@ -125,7 +137,7 @@ incmasterw(const char *arg) {
                masterw = MASTERWIDTH;
        else {
                i = atoi(arg);
-               if(waw * (masterw + i) / 1000 >= waw - 2 * BORDERPX
+               if(waw * (masterw + i) / 1000 >= waw - 2 * BORDERPX 
                || waw * (masterw + i) / 1000 <= 2 * BORDERPX)
                        return;
                masterw += i;
@@ -199,11 +211,9 @@ setlayout(const char *arg) {
        int i;
 
        if(!arg) {
-               for(i = 0; i < nlayouts && lt != &layout[i]; i++);
-               if(i == nlayouts - 1)
-                       lt = &layout[0];
-               else
-                       lt = &layout[++i];
+               lt++;
+               if(lt == layout + nlayouts)
+                       lt = layout;
        }
        else {
                i = atoi(arg);
@@ -218,6 +228,16 @@ setlayout(const char *arg) {
 }
 
 void
+togglebar(const char *arg) {
+       if(bpos == BarOff)
+               bpos = (BARPOS == BarOff) ? BarTop : BARPOS;
+       else
+               bpos = BarOff;
+       updatebarpos();
+       lt->arrange();
+}
+
+void
 togglemax(const char *arg) {
        XEvent ev;
 
@@ -238,13 +258,10 @@ togglemax(const char *arg) {
 
 void
 zoom(const char *arg) {
-       unsigned int n;
        Client *c;
 
-       if(!sel || lt->arrange != tile || sel->isfloating)
+       if(!sel || lt->arrange == floating || sel->isfloating)
                return;
-       for(n = 0, c = nexttiled(clients); c; c = nexttiled(c->next))
-               n++;
        if((c = sel) == nexttiled(clients))
                if(!(c = nexttiled(c->next)))
                        return;